/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var FAS = {
  
  // nsISupports
  QueryInterface: function FAS_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  observe: function FAS_observe(subject, topic, data) {
    
    if (topic != "netscape-fas")
      return;
  
    // do update requested by back-end
    var self = this;  
    window.setTimeout(function() { self.update(); }, 0);
  },
  
  get button() { return document.getElementById("fas_button"); },
  get command() { return document.getElementById("fas_panel"); },
  get currentURL() { return window.content.document.location.href; },
    
  _domain: "extensions.netscape.fas.",
  _branch: null,
  get branch() { return this._branch; },
  set branch(val) { this._branch = val; },
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },
  
  _loaded: false,
  get loaded() { return this._loaded; },
  set loaded(val) { this._loaded = val; },
  
  onLoad: function FAS_onLoad() {
    // already loaded
    if (this.loaded)
      return;
    this.loaded = true;
        
    // Get the fas service
    this.fasService = Cc["@netscape.com/fas/service;1"].
                      getService(Ci.nsIFASService);    
    
    // Get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-fas", false);
     
    // get the pref branch
    this.branch = Cc["@mozilla.org/preferences-service;1"].
                  getService(Ci.nsIPrefService).
                  getBranch(this._domain);
     
    // Update the UI
    var self = this;
    window.setTimeout(function() { self.update(); }, 100);                      
  },
  
  onUnload: function FAS_onUnload() {
    // never loaded
    if (!this.loaded)
      return;
    this.loaded = false;
       
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-fas");

    // remove variables
    this.obService = null;
    this.fasService = null;  
    this.branch = null;   
  },
  
  getString: function FAS_getString(name, replace) { 
  
    // get the string bundle
    var bundle = document.getElementById("fas_bundle");
    if (!bundle)
      return null;
    
    // return the string  
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  },
      
  update: function FAS_update() {

    // set disabled attribute on the command and button
    var disabled = this.branch.getBoolPref("disabled");
    var button = this.button;
    var command = this.command;
    if (disabled) {
      command.setAttribute("hidden", "true");
      command.setAttribute("disabled", "true");
      if (button) {
        button.setAttribute("hidden", "true");
        button.setAttribute("disabled", "true");
      }
      if (command.getAttribute("checked") == "true")
        toggleSidebar("fas_panel");
    } else {
      command.removeAttribute("hidden");
      command.removeAttribute("disabled");
      if (button) {
        button.removeAttribute("hidden");
        button.removeAttribute("disabled");
      }
    }
    
    // Get the FAS Button
    if (!button || button.hasAttribute("disabled"))
      return;

    // Update with paused
    const FAS_PREFIX = "fas.overlay.tooltip.";    
    button.removeAttribute("new");     
    if (this.fasService.paused) {
      command.setAttribute("disabled", "true");      
      button.setAttribute("disabled", "true"); 
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "paused"));

    // Update with not signed in    
    } else if (!this.fasService.isSignedIn) {
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "login"));
    
    // Update with no activity
    } else if (!this.fasService.hasActivity) {
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "none"));

    // Update with activity
    } else {
      var count = String(this.fasService.totalCount);
      button.setAttribute("new", "true");
      if (count == "1")
        button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "single", [count]));
      else
        button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "plural", [count]));
    }  
  },
  
  checkActivity: function FAS_checkActivity() {
    this.fasService.checkActivity();
  },
  
  checkURL: function FAS_checkURL(URL) {
    if (!this.loaded)
      this.onLoad();    
    this.fasService.checkURL(URL);
  },
    
  open: function FAS_open(event) {
    if (event.button !== 0)
      return;
      
    var el = this.command;
    var sidebar = document.getElementById("sidebar");
    if (el.getAttribute("checked") == "true")
      sidebar.contentWindow.FASPanel.reload();
    else
      toggleSidebar("fas_panel", true);
  }
};